from node import Node


class GenericNetwork:
    def __init__(self):
        # key: node identifier, value: Node-object
        self.nodes = {}

    def read_from_tsv(self, file_path):
        """
        Reads white-space-separated files that contain two or more columns. The first two columns contain the
        identifiers of two nodes that have an undirected edge. The two nodes are added to the network.
        :param file_path: path to the file
        """
        # clear the prior content of the network
        self.nodes = {}

        # open the file for reading
        with open(file_path, 'r') as file:
            # iterate over the lines in the file
            for line in file:
                #
                columns = line.split()

                # skip lines that do not have two node identifiers
                if len(columns) < 2:
                    continue

                # create the two nodes and remove potential whitespace such as new-line from their identifiers
                node_1 = Node(columns[0].strip())
                node_2 = Node(columns[1].strip())
                # add the nodes and the edge between them to the network
                self.add_node(node_1)
                self.add_node(node_2)
                self.add_edge(node_1, node_2)

    def add_node(self, node):
        """
        Adds the specified node to the network.
        :param node: Node-object
        """
        if node.ident not in self.nodes.keys():
            self.nodes[node.ident] = node

    def add_edge(self, node_1, node_2):
        """
        Adds an (undirected) edge between the two specified nodes.
        :param node_1: Node-object
        :param node_2: Node-object
        :raises: KeyError if either node is not in the network
        """
        # raise an error if the nodes are not in the network
        if node_1.ident not in self.nodes.keys():
            raise KeyError('There is no node in the network with identifier:', node_1)
        if node_2.ident not in self.nodes.keys():
            raise KeyError('There is no node in the network with identifier:', node_2)

        # add the (undirected) edge
        self.nodes[node_1.ident].add_edge(node_2)
        self.nodes[node_2.ident].add_edge(node_1)

    def get_node(self, identifier):
        """
        :param identifier: node identifier
        :return: Node-object corresponding to the given node identifier, if the node is in the network
        :raises: KeyError if there is no node with that identifier in the network
        """
        if identifier not in self.nodes.keys():
            raise KeyError('There is no node in the network with identifier:', identifier)
        return self.nodes[identifier]

    def has_edge(self, node_1, node_2):
        """
        :param node_1: Node-object
        :param node_2: Node-object
        :return: True if the two nodes have an (undirected) edge, False otherwise
        :raises: KeyError if either node is not in the network
        """
        # raise an error if the nodes are not in the network
        if node_1.ident not in self.nodes.keys():
            raise KeyError('There is no node in the network with identifier:', node_1)
        if node_2.ident not in self.nodes.keys():
            raise KeyError('There is no node in the network with identifier:', node_2)

        return node_1.has_edge_to(node_2) and node_2.has_edge_to(node_1)

    def size(self):
        """
        :return: number of nodes in the network
        """
        return len(self.nodes.keys())

    def max_degree(self):
        """
        :return: highest node degree in the network, 0 if there are no nodes in the network
        """
        return max([node.degree() for node in self.nodes.values()], default=0)
